@@ -853,7 +853,7 @@ std::unique_ptr<IOBuf> LZ4FrameCodec::doUncompress(
   }
   data = nullptr;
   // Select decompression options
-  LZ4F_decompressOptions_t options;
+  LZ4F_decompressOptions_t options{};
   options.stableDst = 1;
   // Select blockSize and growthSize for the IOBufQueue
   IOBufQueue queue(IOBufQueue::cacheChainLength());
@@ -1106,12 +1106,12 @@ bool LZMA2StreamCodec::canUncompress(
 
 std::unique_ptr<Codec> LZMA2StreamCodec::createCodec(
     int level, CodecType type) {
-  return make_unique<LZMA2StreamCodec>(level, type);
+  return std::make_unique<LZMA2StreamCodec>(level, type);
 }
 
 std::unique_ptr<StreamCodec> LZMA2StreamCodec::createStream(
     int level, CodecType type) {
-  return make_unique<LZMA2StreamCodec>(level, type);
+  return std::make_unique<LZMA2StreamCodec>(level, type);
 }
 
 LZMA2StreamCodec::LZMA2StreamCodec(int level, CodecType type)
