@@ -14,12 +14,12 @@
  * limitations under the License.
  */
 
-#include <folly/portability/Config.h>
-
 // AtomicSharedPtr-detail.h only works with libstdc++, so skip these tests for
 // other vendors
 // PackedSyncPtr requires x64, ppc64 or aarch64, skip these tests for
 // other arches
+#include <folly/Portability.h>
+#include <folly/portability/Config.h>
 #if defined(__GLIBCXX__) && (FOLLY_X64 || FOLLY_PPC64 || FOLLY_AARCH64)
 
 #include <atomic>
@@ -28,7 +28,7 @@
 #include <vector>
 
 #include <folly/Benchmark.h>
-#include <folly/Portability.h>
+#include <folly/Optional.h>
 #include <folly/concurrency/AtomicSharedPtr.h>
 #include <folly/concurrency/CoreCachedSharedPtr.h>
 #include <folly/experimental/ReadMostlySharedPtr.h>
@@ -128,7 +128,7 @@ namespace {
 template <class Holder>
 void testAliasingCornerCases() {
   {
-    Holder h;
+    folly::Optional<Holder> h(folly::in_place);
     auto p1 = std::make_shared<int>(1);
     std::weak_ptr<int> w1 = p1;
     // Aliasing constructor, p2 is nullptr but still manages the object in p1.
@@ -138,15 +138,14 @@ void testAliasingCornerCases() {
     EXPECT_FALSE(w1.expired());
 
     // Pass the ownership to the Holder.
-    h.reset(p2);
+    h->reset(p2);
     p2.reset();
 
     // Object should still be alive.
     EXPECT_FALSE(w1.expired());
 
-    // And resetting will destroy it.
+    // And destroying the holder will retire it.
     h.reset();
-    folly::hazptr_cleanup();
     EXPECT_TRUE(w1.expired());
   }
 
