@@ -93,17 +93,27 @@
 #include <type_traits>
 #include <utility>
 #include <vector>
+
 #include <folly/Range.h>
 #include <folly/ScopeGuard.h>
 #include <folly/Traits.h>
 #include <folly/Utility.h>
 #include <folly/container/Iterator.h>
+#include <folly/functional/Invoke.h>
 #include <folly/lang/Exception.h>
 #include <folly/memory/MemoryResource.h>
 #include <folly/portability/Builtins.h>
 #include <folly/small_vector.h>
 
 namespace folly {
+template <
+    typename Key,
+    typename Value,
+    typename Compare,
+    typename Allocator,
+    typename GrowthPolicy,
+    typename Container>
+class heap_vector_map;
 
 namespace detail {
 
@@ -267,7 +277,7 @@ typename Container::difference_type distance(
   auto offset = getStartOffsetToScan(start, end);
   calculateDistance(offset, size_type(0));
   // Handle start == end()
-  if (start >= size) {
+  if (start > size) {
     dist *= -1;
   }
 
@@ -340,7 +350,12 @@ void heapify(Container& cont) {
     if (next == index) {
       continue;
     }
-    auto tmp = std::move(cont[index]);
+    // Subtlety: operator[] returns a Container::reference. Because
+    // Container::reference can be a proxy, using bare `auto` is not
+    // sufficient to remove the "reference nature" of
+    // Container::reference and force a move out of the container;
+    // instead, we need Container::value_type.
+    typename Container::value_type tmp = std::move(cont[index]);
     rotate(next, index);
     cont[next] = std::move(tmp);
   }
@@ -629,38 +644,38 @@ Container&& as_sorted_unique(Container&& container, Compare const& comp) {
 }
 
 // class value_compare_map is used to compare map elements.
-template <class Compare, class value_type>
+template <class Compare>
 struct value_compare_map : Compare {
-  bool operator()(const value_type& a, const value_type& b) const {
-    return Compare::operator()(a.first, b.first);
+  template <typename... value_type>
+  auto operator()(const value_type&... a) const
+      noexcept(is_nothrow_invocable_v<const Compare&, decltype((a.first))...>)
+          -> invoke_result_t<const Compare&, decltype((a.first))...> {
+    return Compare::operator()(a.first...);
   }
 
-  using first_type = typename value_type::first_type;
-  first_type& getKey(const value_type& a) const {
-    return const_cast<value_type&>(a).first;
+  template <typename value_type>
+  const auto& getKey(const value_type& a) const noexcept {
+    return a.first;
   }
 
-  first_type& getKey(const value_type& a) {
-    return const_cast<value_type&>(a).first;
-  }
-
-  explicit value_compare_map(const Compare& c) : Compare(c) {}
+  explicit value_compare_map(const Compare& c) noexcept(
+      std::is_nothrow_copy_constructible<Compare>::value)
+      : Compare(c) {}
 };
 
 // wrapper class value_compare_set for set elements.
-template <class Compare, class value_type>
+template <class Compare>
 struct value_compare_set : Compare {
-  bool operator()(const value_type& a, const value_type& b) const {
-    return Compare::operator()(a, b);
+  using Compare::operator();
+
+  template <typename value_type>
+  value_type& getKey(value_type& a) const noexcept {
+    return a;
   }
 
-  value_type& getKey(const value_type& a) const {
-    return const_cast<value_type&>(a);
-  }
-
-  value_type& getKey(const value_type& a) { return const_cast<value_type&>(a); }
-
-  explicit value_compare_set(const Compare& c) : Compare(c) {}
+  explicit value_compare_set(const Compare& c) noexcept(
+      std::is_nothrow_copy_constructible<Compare>::value)
+      : Compare(c) {}
 };
 
 /**
@@ -687,7 +702,7 @@ template <
     class GrowthPolicy = void,
     class Container = std::vector<T, Allocator>,
     class KeyT = T,
-    class ValueCompare = value_compare_set<Compare, T>>
+    class ValueCompare = value_compare_set<Compare>>
 class heap_vector_container : growth_policy_wrapper<GrowthPolicy> {
  protected:
   growth_policy_wrapper<GrowthPolicy>& get_growth_policy() { return *this; }
@@ -716,11 +731,11 @@ class heap_vector_container : growth_policy_wrapper<GrowthPolicy> {
   struct heap_iterator {
     using iterator_category = std::random_access_iterator_tag;
     using size_type = typename Container::size_type;
-    using difference_type = typename Container::difference_type;
-    using value_type = typename Container::value_type;
-    using pointer = typename Container::pointer;
-    using reference = typename Container::reference;
-    using const_reference = typename Container::const_reference;
+    using difference_type =
+        typename std::iterator_traits<Iter>::difference_type;
+    using value_type = typename std::iterator_traits<Iter>::value_type;
+    using pointer = typename std::iterator_traits<Iter>::pointer;
+    using reference = typename std::iterator_traits<Iter>::reference;
 
     heap_iterator() = default;
     template <typename C>
@@ -835,14 +850,38 @@ class heap_vector_container : growth_policy_wrapper<GrowthPolicy> {
       return heap_vector_detail::distance(*cont_, offset1, offset0);
     }
 
-    reference operator*() { return const_cast<reference>(*ptr_); }
-    reference operator*() const { return const_cast<reference>(*ptr_); }
-    pointer operator->() { return &const_cast<reference>(*ptr_); }
-    pointer operator->() const { return &const_cast<reference>(*ptr_); }
+    reference operator*() const { return *ptr_; }
+    pointer operator->() const {
+      if constexpr (std::is_pointer_v<Iter>) {
+        return ptr_;
+      } else {
+        return ptr_.operator->();
+      }
+    }
 
    protected:
     template <typename I2>
     friend struct heap_iterator;
+
+    template <
+        typename T2,
+        typename Compare2,
+        typename Allocator2,
+        typename GrowthPolicy2,
+        typename Container2,
+        typename KeyT2,
+        typename ValueCompare2>
+    friend class heap_vector_container;
+
+    template <
+        typename Key2,
+        typename Value2,
+        typename Compare2,
+        typename Allocator2,
+        typename GrowthPolicy2,
+        typename Container2>
+    friend class ::folly::heap_vector_map;
+
     Iter ptr_;
     Container* cont_;
   };
@@ -951,6 +990,8 @@ class heap_vector_container : growth_policy_wrapper<GrowthPolicy> {
 
   Allocator get_allocator() const { return m_.cont_.get_allocator(); }
 
+  const Container& get_container() const noexcept { return m_.cont_; }
+
   heap_vector_container& operator=(const heap_vector_container& other) =
       default;
 
@@ -1023,7 +1064,7 @@ class heap_vector_container : growth_policy_wrapper<GrowthPolicy> {
   std::pair<iterator, bool> insert(const value_type& value) {
     iterator it = lower_bound(m_.getKey(value));
     if (it == end() || value_comp()(value, *it)) {
-      auto offset = &*it - &*m_.cont_.begin();
+      auto offset = it.ptr_ - m_.cont_.begin();
       get_growth_policy().increase_capacity(*this, it);
       m_.cont_.push_back(value);
       offset = heap_vector_detail::insert(offset, m_.cont_);
@@ -1036,7 +1077,7 @@ class heap_vector_container : growth_policy_wrapper<GrowthPolicy> {
   std::pair<iterator, bool> insert(value_type&& value) {
     iterator it = lower_bound(m_.getKey(value));
     if (it == end() || value_comp()(value, *it)) {
-      auto offset = &*it - &*m_.cont_.begin();
+      auto offset = it.ptr_ - m_.cont_.begin();
       get_growth_policy().increase_capacity(*this, it);
       m_.cont_.push_back(std::move(value));
       offset = heap_vector_detail::insert(offset, m_.cont_);
@@ -1114,13 +1155,13 @@ class heap_vector_container : growth_policy_wrapper<GrowthPolicy> {
     if (it == end()) {
       return 0;
     }
-    heap_vector_detail::erase(&*it - &*m_.cont_.begin(), m_.cont_);
+    heap_vector_detail::erase(it.ptr_ - m_.cont_.begin(), m_.cont_);
     return 1;
   }
 
   iterator erase(const_iterator it) {
     auto offset =
-        heap_vector_detail::erase(&*it - &*m_.cont_.begin(), m_.cont_);
+        heap_vector_detail::erase(it.ptr_ - m_.cont_.begin(), m_.cont_);
     iterator ret = end();
     ret = m_.cont_.begin() + offset;
     return ret;
@@ -1315,10 +1356,8 @@ class heap_vector_container : growth_policy_wrapper<GrowthPolicy> {
   template <typename Self, typename K>
   static self_iterator_t<Self> lower_bound(Self& self, K const& key) {
     auto c = self.key_comp();
-    auto cmp = [&](value_type const& a) { return c(self.m_.getKey(a), key); };
-    auto reverseCmp = [&](value_type const& a) {
-      return c(key, self.m_.getKey(a));
-    };
+    auto cmp = [&](auto const& a) { return c(self.m_.getKey(a), key); };
+    auto reverseCmp = [&](auto const& a) { return c(key, self.m_.getKey(a)); };
     auto offset =
         heap_vector_detail::lower_bound(self.m_.cont_, cmp, reverseCmp);
     self_iterator_t<Self> ret = self.end();
@@ -1329,7 +1368,7 @@ class heap_vector_container : growth_policy_wrapper<GrowthPolicy> {
   template <typename Self, typename K>
   static self_iterator_t<Self> upper_bound(Self& self, K const& key) {
     auto c = self.key_comp();
-    auto cmp = [&](value_type const& a) { return c(key, self.m_.getKey(a)); };
+    auto cmp = [&](auto const& a) { return c(key, self.m_.getKey(a)); };
     auto offset = heap_vector_detail::upper_bound(self.m_.cont_, cmp);
     self_iterator_t<Self> ret = self.end();
     ret = self.m_.cont_.begin() + offset;
@@ -1364,7 +1403,7 @@ class heap_vector_set
           GrowthPolicy,
           Container,
           T,
-          detail::heap_vector_detail::value_compare_set<Compare, T>> {
+          detail::heap_vector_detail::value_compare_set<Compare>> {
  private:
   using heap_vector_container =
       detail::heap_vector_detail::heap_vector_container<
@@ -1374,7 +1413,7 @@ class heap_vector_set
           GrowthPolicy,
           Container,
           T,
-          detail::heap_vector_detail::value_compare_set<Compare, T>>;
+          detail::heap_vector_detail::value_compare_set<Compare>>;
 
  public:
   using heap_vector_container::heap_vector_container;
@@ -1438,8 +1477,7 @@ class heap_vector_map
           GrowthPolicy,
           Container,
           Key,
-          detail::heap_vector_detail::
-              value_compare_map<Compare, typename Container::value_type>> {
+          detail::heap_vector_detail::value_compare_map<Compare>> {
  public:
   using key_type = Key;
   using mapped_type = Value;
@@ -1452,8 +1490,7 @@ class heap_vector_map
   using const_reference = typename Container::const_reference;
   using difference_type = typename Container::difference_type;
   using size_type = typename Container::size_type;
-  using value_compare =
-      detail::heap_vector_detail::value_compare_map<Compare, value_type>;
+  using value_compare = detail::heap_vector_detail::value_compare_map<Compare>;
 
  protected:
   using heap_vector_container =
@@ -1501,7 +1538,7 @@ class heap_vector_map
   mapped_type& operator[](const key_type& key) {
     iterator it = lower_bound(key);
     if (it == end() || key_comp()(key, it->first)) {
-      auto offset = &*it - &*m_.cont_.begin();
+      auto offset = it.ptr_ - m_.cont_.begin();
       get_growth_policy().increase_capacity(*this, it);
       m_.cont_.emplace_back(key, mapped_type());
       offset = detail::heap_vector_detail::insert(offset, m_.cont_);
@@ -1550,7 +1587,10 @@ template <
     typename Key,
     typename Value,
     typename SizeType = uint32_t,
-    class Container = folly::small_vector<std::pair<Key, Value>, 0, SizeType>,
+    class Container = folly::small_vector<
+        std::pair<Key, Value>,
+        0,
+        folly::small_vector_policy::policy_size_type<SizeType>>,
     typename = std::enable_if_t<std::is_integral<Key>::value>>
 class small_heap_vector_map : public folly::heap_vector_map<
                                   Key,
@@ -1572,7 +1612,7 @@ class small_heap_vector_map : public folly::heap_vector_map<
   using difference_type = typename Container::difference_type;
   using size_type = typename Container::size_type;
   using value_compare =
-      detail::heap_vector_detail::value_compare_map<std::less<Key>, value_type>;
+      detail::heap_vector_detail::value_compare_map<std::less<Key>>;
 
  private:
   using heap_vector_map = folly::
