@@ -174,7 +174,15 @@ TEST(EvictingCacheMap, PruneHookTest) {
     sum += k;
   };
 
+  EXPECT_FALSE(map.getPruneHook());
   map.setPruneHook(pruneCb);
+  EXPECT_TRUE(map.getPruneHook());
+  {
+    int v = 42;
+    map.getPruneHook()(42, std::move(v));
+  }
+  EXPECT_EQ(42, sum);
+  sum = 0;
 
   for (int i = 0; i < 100; i++) {
     map.set(i, i);
@@ -260,6 +268,33 @@ TEST(EvictingCacheMap, PruneHookTest) {
   }
   EXPECT_EQ((89 * 90) / 2, sum);
   sum = 0;
+
+  // Erase does not call prune hook (NOTE: possible source of usage bugs)
+  map.erase(99);
+
+  EXPECT_EQ(0, sum);
+
+  // But can provide your own hook
+  map.erase(98, pruneCb);
+
+  EXPECT_EQ(98, sum);
+  sum = 0;
+
+  // And with iterator
+  auto it1 = map.find(96);
+  auto it2 = map.find(97);
+
+  auto it3 = map.erase(it2, pruneCb);
+  EXPECT_EQ(it1, it3);
+  EXPECT_EQ(97, sum);
+  sum = 0;
+
+  // Destructor does not call prune hook (NOTE: possibly source of usage bugs)
+  map.~EvictingCacheMap<int, int>();
+  // Re-enter clean state
+  new (&map) EvictingCacheMap<int, int>(0);
+
+  EXPECT_EQ(0, sum);
 }
 
 TEST(EvictingCacheMap, SetMaxSize) {
@@ -665,12 +700,21 @@ TEST(EvictingCacheMap, MoveTest) {
     EXPECT_EQ(i, map.get(i));
   }
 
+  // Move to empty
   EvictingCacheMap<int, int> map2 = std::move(map);
   EXPECT_TRUE(map.empty());
   for (int i = 0; i < nItems; i++) {
     EXPECT_TRUE(map2.exists(i));
     EXPECT_EQ(i, map2.get(i));
   }
+
+  // Move to non-empty
+  EvictingCacheMap<int, int> map3(1);
+  map3.set(1, 1);
+  EXPECT_EQ(1, map3.size());
+  map3 = std::move(map2);
+  EXPECT_TRUE(map2.empty());
+  EXPECT_EQ(nItems, map3.size());
 }
 
 TEST(EvictingCacheMap, CustomKeyEqual) {
@@ -696,6 +740,17 @@ TEST(EvictingCacheMap, CustomKeyEqual) {
   }
 }
 
+TEST(EvictingCacheMap, InvalidHashPartlyUsable) {
+  // Some uses of EvictingCacheMap only use constructor+destructor in a header
+  // file where the hasher is invalid. (Only fully defined in cpp file.)
+  struct BadHash {};
+  struct BadEq {};
+  EvictingCacheMap<int, int, BadHash, BadEq> map{42};
+  // Also move ctor and operator
+  EvictingCacheMap<int, int, BadHash, BadEq> map2{std::move(map)};
+  map = std::move(map2);
+}
+
 TEST(EvictingCacheMap, IteratorConversion) {
   using type = EvictingCacheMap<int, int>;
   using i = type::iterator;
@@ -764,3 +819,16 @@ TEST(EvictingCacheMap, HeterogeneousAccess) {
   }
   EXPECT_TRUE(map.empty());
 }
+
+TEST(EvictingCacheMap, ApproximateEntryMemUsage) {
+  // Entry (without weight) should be
+  // * two pointers for LRU list
+  // * sizeof(key) and sizeof(value)
+  // * roughly 1.5 pointers for F14 index
+  // And sizeof(std::unique_ptr<char[]>) should usually be size of raw pointer
+  EXPECT_EQ(sizeof(std::unique_ptr<char[]>), sizeof(char*));
+  EXPECT_LE(
+      (EvictingCacheMap<uint64_t, std::unique_ptr<char[]>>::
+           kApproximateEntryMemUsage),
+      48U);
+}
