@@ -350,6 +350,7 @@ TEST(HeapVectorTypes, FromVector) {
 
     heap_vector_map<int, float> m(std::move(vec));
 
+    // NOLINTNEXTLINE(bugprone-use-after-move)
     EXPECT_EQ(vec.size(), 0);
     EXPECT_EQ(m.size(), 3);
     EXPECT_EQ(m[1], 1.0f);
@@ -364,6 +365,7 @@ TEST(HeapVectorTypes, FromVector) {
 
     small_heap_vector_map<int, float> m(std::move(vec));
 
+    // NOLINTNEXTLINE(bugprone-use-after-move)
     EXPECT_EQ(vec.size(), 0);
     EXPECT_EQ(m.size(), 3);
     EXPECT_EQ(m[1], 1.0f);
@@ -636,7 +638,11 @@ TEST(HeapVectorTypes, Sizes) {
       sizeof(std::vector<std::pair<int, int>>));
   EXPECT_EQ(
       sizeof(small_heap_vector_map<int, int>),
-      sizeof(folly::small_vector<std::pair<int, int>, 0, uint32_t>));
+      sizeof( //
+          folly::small_vector<
+              std::pair<int, int>,
+              0,
+              folly::small_vector_policy::policy_size_type<uint32_t>>));
 
   using SetT = heap_vector_set<
       int,
@@ -684,6 +690,49 @@ TEST(HeapVectorTypes, Iterators) {
   small_heap_vector_map<int, int>::const_iterator cmapI2(mapI2);
 }
 
+TEST(HeapVectorTypes, IteratorsCombinatorics) {
+  heap_vector_set<size_t> c;
+  ASSERT_EQ(0, c.size());
+  ASSERT_EQ(c.begin(), c.end());
+  ASSERT_EQ(c.find(0), c.end());
+  for (size_t i = 0; i < 36; ++i) {
+    auto [ii, inserted] = c.insert(i);
+    auto bi = c.begin();
+    auto ei = c.end();
+    ASSERT_EQ(i, *ii);
+    ASSERT_EQ(ii, c.find(i));
+    ASSERT_TRUE(inserted);
+    ASSERT_EQ(i + 1, c.size());
+    ASSERT_NE(bi, ei);
+    EXPECT_EQ(c.size(), std::distance(bi, ei));
+    EXPECT_EQ(ei, bi + i + 1);
+    EXPECT_EQ(1, std::distance(bi + i, ei));
+    EXPECT_NE(ii, ei);
+    EXPECT_EQ(ii, bi + i);
+    EXPECT_EQ(bi, ii - i);
+    EXPECT_EQ(i, std::distance(bi, ii));
+    EXPECT_EQ(ei, ii + 1);
+    EXPECT_EQ(ii, ei - 1);
+    EXPECT_EQ(1, std::distance(ii, ei));
+    for (size_t j = 0; j <= i; ++j) {
+      auto ji = c.find(j);
+      ASSERT_NE(ji, ei);
+      ASSERT_EQ(j, *ji);
+      EXPECT_EQ(bi, ji - j);
+      EXPECT_EQ(ji, bi + j);
+      for (size_t k = 0; k <= i; ++k) {
+        auto ki = c.find(k);
+        ASSERT_NE(ki, ei);
+        EXPECT_EQ(j == k, ji == ki);
+        EXPECT_EQ(ji, ki + (int(j) - int(k)));
+        EXPECT_EQ(ji, ki - (int(k) - int(j)));
+        EXPECT_EQ(int(k) - int(j), ki - ji);
+        EXPECT_EQ(int(k) - int(j), std::distance(ji, ki));
+      }
+    }
+  }
+}
+
 TEST(HeapVectorTypes, InitializerLists) {
   heap_vector_set<int> empty_initialized_set{};
   EXPECT_TRUE(empty_initialized_set.empty());
@@ -743,13 +792,6 @@ TEST(HeapVectorTypes, CustomCompare) {
     }
     check_invariant(m);
   }
-  {
-    small_heap_vector_map<int, float, less_invert<int>> m;
-    for (int i = 0; i < 200; ++i) {
-      m[i] = 12.0f;
-    }
-    check_invariant(m);
-  }
 }
 
 TEST(HeapVectorTypes, GrowthPolicy) {
@@ -1193,8 +1235,11 @@ TEST(HeapVectorTypes, TestMapCreationFromVector) {
 TEST(HeapVectorTypes, TestMapCreationFromVectorSmall) {
   // TODO: Add a constructor to steal std::vector. For small_heap_vector_map
   // it is better to steal from small_vector.
-  folly::small_vector<std::pair<int, int>, 0, uint32_t> vec = {
-      {3, 1}, {1, 5}, {-1, 2}, {5, 3}, {0, 3}};
+  folly::small_vector<
+      std::pair<int, int>,
+      0,
+      folly::small_vector_policy::policy_size_type<uint32_t>>
+      vec = {{3, 1}, {1, 5}, {-1, 2}, {5, 3}, {0, 3}};
   small_heap_vector_map<int, int> vmap(std::move(vec));
   check_invariant(vmap);
   auto contents = std::vector<std::pair<int, int>>(vmap.begin(), vmap.end());
@@ -1482,6 +1527,7 @@ TEST(HeapVectorTypes, TestPmrMoveConstructSameAlloc) {
     auto d = s1.iterate().data();
 
     pmr::heap_vector_set<int> s2(std::move(s1), a2);
+    // NOLINTNEXTLINE(bugprone-use-after-move)
     EXPECT_EQ(s1.get_allocator(), s2.get_allocator());
     EXPECT_EQ(s2.iterate().data(), d);
     EXPECT_EQ(s2.count(42), 1);
@@ -1492,6 +1538,7 @@ TEST(HeapVectorTypes, TestPmrMoveConstructSameAlloc) {
     auto d = m1.iterate().data();
 
     pmr::heap_vector_map<int, int> m2(std::move(m1), a2);
+    // NOLINTNEXTLINE(bugprone-use-after-move)
     EXPECT_EQ(m1.get_allocator(), m2.get_allocator());
     EXPECT_EQ(m2.iterate().data(), d);
     EXPECT_EQ(m2.at(42), 42);
@@ -1513,6 +1560,7 @@ TEST(HeapVectorTypes, TestPmrMoveConstructDifferentAlloc) {
     auto d = s1.iterate().data();
 
     pmr::heap_vector_set<int> s2(std::move(s1), a2);
+    // NOLINTNEXTLINE(bugprone-use-after-move)
     EXPECT_NE(s1.get_allocator(), s2.get_allocator());
     EXPECT_NE(s2.iterate().data(), d);
     EXPECT_EQ(s2.count(42), 1);
@@ -1523,6 +1571,7 @@ TEST(HeapVectorTypes, TestPmrMoveConstructDifferentAlloc) {
     auto d = m1.iterate().data();
 
     pmr::heap_vector_map<int, int> m2(std::move(m1), a2);
+    // NOLINTNEXTLINE(bugprone-use-after-move)
     EXPECT_NE(m1.get_allocator(), m2.get_allocator());
     EXPECT_NE(m2.iterate().data(), d);
     EXPECT_EQ(m2.at(42), 42);
@@ -1688,3 +1737,12 @@ TEST(HeapVectorTypes, TestIterator) {
     EXPECT_EQ(I->first, --i);
   }
 }
+
+TEST(HeapVectorTypes, TestGetContainer) {
+  heap_vector_map<int, int> m;
+  EXPECT_TRUE(m.get_container().empty());
+  small_heap_vector_map<int, int> m2;
+  EXPECT_TRUE(m2.get_container().empty());
+  heap_vector_set<int> s;
+  EXPECT_TRUE(s.get_container().empty());
+}
