@@ -907,6 +907,15 @@ struct dynamic {
   template <class T>
   iterator insert(const_iterator pos, T&& value);
 
+  /**
+   * Inserts elements from range [first, last) before pos into an array.
+   * Throws if the type is not an array.
+   *
+   * Invalidates iterators.
+   */
+  template <class InputIt>
+  iterator insert(const_iterator pos, InputIt first, InputIt last);
+
   /**
    * Merge objects.
    *
