@@ -31,16 +31,27 @@
 
 using namespace folly;
 
-DEFINE_uint32(
+FOLLY_GFLAGS_DEFINE_uint32(
     folly_global_io_executor_threads,
     0,
     "Number of threads global IOThreadPoolExecutor will create");
 
-DEFINE_uint32(
+FOLLY_GFLAGS_DEFINE_uint32(
     folly_global_cpu_executor_threads,
     0,
     "Number of threads global CPUThreadPoolExecutor will create");
 
+FOLLY_GFLAGS_DEFINE_bool(
+    folly_global_cpu_executor_use_throttled_lifo_sem,
+    true,
+    "Use ThrottledLifoSem in global CPUThreadPoolExecutor");
+
+FOLLY_GFLAGS_DEFINE_uint32(
+    folly_global_cpu_executor_wake_up_interval_us,
+    0,
+    "If --folly_global_cpu_executor_use_throttled_lifo_sem is true, use this "
+    "wake-up interval (in microseconds) in ThrottledLifoSem");
+
 namespace {
 
 class GlobalTag {};
@@ -59,7 +70,13 @@ Singleton<std::shared_ptr<Executor>, GlobalTag> gImmutableGlobalCPUExecutor([] {
   size_t nthreads = FLAGS_folly_global_cpu_executor_threads;
   nthreads = nthreads ? nthreads : folly::hardware_concurrency();
   return new std::shared_ptr<Executor>(new CPUThreadPoolExecutor(
-      nthreads, std::make_shared<NamedThreadFactory>("GlobalCPUThreadPool")));
+      nthreads,
+      FLAGS_folly_global_cpu_executor_use_throttled_lifo_sem
+          ? CPUThreadPoolExecutor::makeThrottledLifoSemQueue(
+                std::chrono::microseconds{
+                    FLAGS_folly_global_cpu_executor_wake_up_interval_us})
+          : CPUThreadPoolExecutor::makeDefaultQueue(),
+      std::make_shared<NamedThreadFactory>("GlobalCPUThreadPool")));
 });
 
 Singleton<std::shared_ptr<IOExecutor>, GlobalTag> gImmutableGlobalIOExecutor(
