@@ -875,8 +875,11 @@ TYPED_TEST(ThreadPoolExecutorTypedTest, RegistersToExecutorList) {
 
 template <typename TPE>
 static void testUsesNameFromNamedThreadFactory() {
-  auto ntf = std::make_shared<NamedThreadFactory>("my_executor");
-  TPE tpe(10, ntf);
+  // Verify that the name is propagated even if the NamedThreadFactory is
+  // wrapped.
+  auto tf = std::make_shared<InitThreadFactory>(
+      std::make_shared<NamedThreadFactory>("my_executor"), [] {});
+  TPE tpe(10, tf);
   EXPECT_EQ("my_executor", tpe.getName());
 }
 
