@@ -211,6 +211,8 @@ class IoUringBackend : public EventBaseBackendBase {
   }
 
   // from EventBaseBackendBase
+  int getPollableFd() const override { return ioRing_.ring_fd; }
+
   event_base* getEventBase() override { return nullptr; }
 
   int eb_event_base_loop(int flags) override;
@@ -295,6 +297,14 @@ class IoUringBackend : public EventBaseBackendBase {
 
   void queueClose(int fd, FileOpCallback&& cb);
 
+  void queueStatx(
+      int dirfd,
+      const char* pathname,
+      int flags,
+      unsigned int mask,
+      struct statx* statxbuf,
+      FileOpCallback&& cb);
+
   void queueFallocate(
       int fd, int mode, off_t offset, off_t len, FileOpCallback&& cb);
 
@@ -904,6 +914,34 @@ class IoUringBackend : public EventBaseBackendBase {
     }
   };
 
+  struct FStatxIoSqe : public FileOpIoSqe {
+    FStatxIoSqe(
+        IoUringBackend* backend,
+        int dfd,
+        const char* pathname,
+        int flags,
+        unsigned int mask,
+        struct statx* statxbuf,
+        FileOpCallback&& cb)
+        : FileOpIoSqe(backend, dfd, std::move(cb)),
+          path_(pathname),
+          flags_(flags),
+          mask_(mask),
+          statxbuf_(statxbuf) {}
+
+    ~FStatxIoSqe() override = default;
+
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
+      ::io_uring_prep_statx(sqe, fd_, path_, flags_, mask_, statxbuf_);
+      ::io_uring_sqe_set_data(sqe, this);
+    }
+
+    const char* path_;
+    int flags_;
+    unsigned int mask_;
+    struct statx* statxbuf_;
+  };
+
   struct FAllocateIoSqe : public FileOpIoSqe {
     FAllocateIoSqe(
         IoUringBackend* backend,
