@@ -20,10 +20,12 @@
 #define FOLLY_EXPERIMENTAL_SYMBOLIZER_ELF_H_
 
 #include <fcntl.h>
+#include <sys/types.h>
 #include <cstdio>
 #include <initializer_list>
 #include <stdexcept>
 #include <system_error>
+#include <unordered_set>
 
 #include <folly/Conv.h>
 #include <folly/Likely.h>
@@ -55,12 +57,22 @@ using ElfRel = FOLLY_ELF_ELFW(Rel);
 using ElfRela = FOLLY_ELF_ELFW(Rela);
 
 // ElfFileId is supposed to uniquely identify any instance of an ELF binary.
-// It does that by using file's inode, dev ID, size and modification time:
-// <dev, inode, size in bytes, mod time>
-// Just using dev, inode is not unique enough, because the file can
-// be overwritten with new contents, but will keep same dev and inode, so
-// we take into account modification time and file size to minimize risk.
-using ElfFileId = std::tuple<dev_t, ino_t, off_t, uint64_t>;
+// It does that by using the file's inode, dev ID, size and modification time
+// (ns): <dev, inode, size in bytes, mod time> Just using dev, inode is not
+// unique enough, because the file can be overwritten with new contents, but
+// will keep same dev and inode, so we take into account modification time and
+// file size to minimize risk.
+struct ElfFileId {
+  dev_t dev;
+  ino_t inode;
+  off_t size;
+  uint64_t mtime;
+};
+
+inline bool operator==(const ElfFileId& lhs, const ElfFileId& rhs) {
+  return lhs.dev == rhs.dev && lhs.inode == rhs.inode && lhs.size == rhs.size &&
+      lhs.mtime == rhs.mtime;
+}
 
 /**
  * ELF file parser.
@@ -249,13 +261,63 @@ class ElfFile {
       std::initializer_list<uint32_t> types = {
           STT_OBJECT, STT_FUNC, STT_GNU_IFUNC}) const noexcept;
 
+  /**
+   * Find multiple symbol definitions by name. Because searching for a symbol is
+   * O(N) this method enables searching for multiple symbols in a single pass.
+   *
+   * Returns a map containing a key for each unique symbol name in the provided
+   * names container. The corresponding value is either Symbol or <nullptr,
+   * nullptr> if the symbol was not found.
+   */
+  template <typename C, typename T = typename C::value_type>
+  std::unordered_map<std::string, Symbol> getSymbolsByName(
+      const C& names,
+      std::initializer_list<uint32_t> types = {
+          STT_OBJECT, STT_FUNC, STT_GNU_IFUNC}) const noexcept {
+    std::unordered_map<std::string, Symbol> result(names.size());
+    for (const std::string& name : names) {
+      result[name] = {nullptr, nullptr};
+    }
+    size_t seenCount = 0;
+
+    auto findSymbol = [&](const folly::symbolizer::ElfShdr& section,
+                          const folly::symbolizer::ElfSym& sym) -> bool {
+      auto symbol = folly::symbolizer::ElfFile::Symbol(&section, &sym);
+      auto name = getSymbolName(symbol);
+      if (name == nullptr) {
+        return false;
+      }
+      auto itr = result.find(name);
+      if (itr != result.end() && itr->second.first == nullptr &&
+          itr->second.second == nullptr) {
+        itr->second = symbol;
+        ++seenCount;
+      }
+      return seenCount == result.size();
+    };
+
+    auto iterSection = [&](const folly::symbolizer::ElfShdr& section) -> bool {
+      iterateSymbolsWithTypes(section, types, [&](const auto& sym) -> bool {
+        return findSymbol(section, sym);
+      });
+      return false;
+    };
+    // Try the .dynsym section first if it exists, it's smaller.
+    iterateSectionsWithType(SHT_DYNSYM, iterSection) ||
+        iterateSectionsWithType(SHT_SYMTAB, iterSection);
+
+    return result;
+  }
+
   /**
    * Get the value of a symbol.
    */
   template <class T>
-  const T& getSymbolValue(const ElfSym* symbol) const noexcept {
+  const T* getSymbolValue(const ElfSym* symbol) const noexcept {
     const ElfShdr* section = getSectionByIndex(symbol->st_shndx);
-    FOLLY_SAFE_CHECK(section, "Symbol's section index is invalid");
+    if (section == nullptr) {
+      return nullptr;
+    }
 
     return valueAt<T>(*section, symbol->st_value);
   }
@@ -268,13 +330,15 @@ class ElfFile {
    * a char* symbol, you'd do something like this:
    *
    *  auto sym = getSymbolByName("someGlobalValue");
-   *  auto addr = getSymbolValue<ElfAddr>(sym.second);
-   *  const char* str = &getAddressValue<const char>(addr);
+   *  auto addrPtr = getSymbolValue<ElfAddr>(sym.second);
+   *  const char* str = getAddressValue<const char>(*addrPtr);
    */
   template <class T>
-  const T& getAddressValue(const ElfAddr addr) const noexcept {
+  const T* getAddressValue(const ElfAddr addr) const noexcept {
     const ElfShdr* section = getSectionContainingAddress(addr);
-    FOLLY_SAFE_CHECK(section, "Address does not refer to existing section");
+    if (section == nullptr) {
+      return nullptr;
+    }
 
     return valueAt<T>(*section, addr);
   }
@@ -282,7 +346,7 @@ class ElfFile {
   /**
    * Retrieve symbol name.
    */
-  const char* getSymbolName(Symbol symbol) const noexcept;
+  const char* getSymbolName(const Symbol& symbol) const noexcept;
 
   /** Find the section containing the given address */
   const ElfShdr* getSectionContainingAddress(ElfAddr addr) const noexcept;
@@ -327,7 +391,7 @@ class ElfFile {
   }
 
   template <class T>
-  const T& valueAt(const ElfShdr& section, const ElfAddr addr) const noexcept {
+  const T* valueAt(const ElfShdr& section, const ElfAddr addr) const noexcept {
     // For exectuables and shared objects, st_value holds a virtual address
     // that refers to the memory owned by sections. Since we didn't map the
     // sections into the addresses that they're expecting (sh_addr), but
@@ -337,14 +401,14 @@ class ElfFile {
     // TODO: For other file types, st_value holds a file offset directly. Since
     //       I don't have a use-case for that right now, just assert that
     //       nobody wants this. We can always add it later.
-    FOLLY_SAFE_CHECK(
-        elfHeader().e_type == ET_EXEC || elfHeader().e_type == ET_DYN ||
-            elfHeader().e_type == ET_CORE,
-        "Only exectuables, shared objects and cores are supported");
-    FOLLY_SAFE_CHECK(
-        addr >= section.sh_addr &&
-            (addr + sizeof(T)) <= (section.sh_addr + section.sh_size),
-        "Address is not contained within the provided segment");
+    if (!(elfHeader().e_type == ET_EXEC || elfHeader().e_type == ET_DYN ||
+          elfHeader().e_type == ET_CORE)) {
+      return nullptr;
+    }
+    if (!(addr >= section.sh_addr &&
+          (addr + sizeof(T)) <= (section.sh_addr + section.sh_size))) {
+      return nullptr;
+    }
 
     // SHT_NOBITS: a section that occupies no space in the file but otherwise
     // resembles SHT_PROGBITS. Although this section contains no bytes, the
@@ -352,10 +416,12 @@ class ElfFile {
     // for zero-initialized data sections like .bss.
     if (section.sh_type == SHT_NOBITS) {
       static T t = {};
-      return t;
+      return &t;
     }
 
-    return at<T>(section.sh_offset + (addr - section.sh_addr));
+    ElfOff offset = section.sh_offset + (addr - section.sh_addr);
+
+    return (offset + sizeof(T) <= length_) ? &at<T>(offset) : nullptr;
   }
 
   static constexpr size_t kFilepathMaxLen = 512;
@@ -371,6 +437,16 @@ class ElfFile {
 } // namespace symbolizer
 } // namespace folly
 
+namespace std {
+template <>
+struct hash<folly::symbolizer::ElfFileId> {
+  size_t operator()(const folly::symbolizer::ElfFileId fileId) const {
+    return folly::hash::hash_combine(
+        fileId.dev, fileId.inode, fileId.size, fileId.mtime);
+  }
+};
+} // namespace std
+
 #include <folly/experimental/symbolizer/Elf-inl.h>
 
 #endif // FOLLY_HAVE_ELF
