@@ -29,7 +29,7 @@
 #endif
 #include <folly/portability/GFlags.h>
 
-DEFINE_string(logging, "", "Logging configuration");
+FOLLY_GFLAGS_DEFINE_string(logging, "", "Logging configuration");
 
 namespace folly {
 const unsigned long kAllFatalSignals =
@@ -113,7 +113,6 @@ Init::Init(int* argc, char*** argv, InitOptions options) {
 }
 
 Init::~Init() {
-  SingletonVault::singleton()->startShutdownTimer();
-  SingletonVault::singleton()->destroyInstances();
+  SingletonVault::singleton()->destroyInstancesFinal();
 }
 } // namespace folly
