@@ -829,8 +829,10 @@ void AsyncServerSocket::setupSocket(NetworkSocket fd, int family) {
 
   // Set reuseaddr to avoid 2MSL delay on server restart
   int one = 1;
-  if (netops::setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one)) !=
-      0) {
+  // AF_UNIX does not support SO_REUSEADDR, setting this would confuse Windows
+  if (family != AF_UNIX &&
+      netops::setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one)) !=
+          0) {
     auto errnoCopy = errno;
     // This isn't a fatal error; just log an error message and continue
     LOG(ERROR) << "failed to set SO_REUSEADDR on async server socket "
