@@ -241,7 +241,7 @@ class EventBase : public TimeoutManager,
     using List = boost::intrusive::list<OnDestructionCallback>;
 
     void schedule(
-        FunctionRef<void(OnDestructionCallback&)> linker,
+        Function<void(OnDestructionCallback&)> linker,
         Function<void(OnDestructionCallback&)> eraser);
 
     friend class EventBase;
@@ -520,8 +520,8 @@ class EventBase : public TimeoutManager,
   void runInLoop(Func c, bool thisIteration = false);
 
   /**
-   * Adds the given callback to a queue of things run before destruction
-   * of current EventBase.
+   * Adds the given callback to a queue of things run on destruction
+   * of current EventBase after the keepalive checks.
    *
    * This allows users of EventBase that run in it, but don't control it, to be
    * notified before EventBase gets destructed.
@@ -537,6 +537,23 @@ class EventBase : public TimeoutManager,
    */
   void runOnDestruction(Func f);
 
+  /**
+   * Adds the given callback to a queue of things run at the start of the
+   * destruction of the current EventBase, before any loop keep-alive handles
+   * are checked.
+   *
+   * Note: will be called from the thread that invoked EventBase destructor,
+   *       before the final run of loop callbacks.
+   */
+  void runOnDestructionStart(OnDestructionCallback& callback);
+
+  /**
+   * Convenience function that allows users to pass in a Function<void()> to be
+   * run at the start of EventBase destruction, before any loop keep-alive
+   * handles are checked.
+   */
+  void runOnDestructionStart(Func f);
+
   /**
    * Adds a callback that will run immediately *before* the event loop.
    * This is very similar to runInLoop(), but will not cause the loop to break:
@@ -822,14 +839,20 @@ class EventBase : public TimeoutManager,
    *
    * @param observer EventHandle's execution observer.
    */
-  void setExecutionObserver(ExecutionObserver* observer) {
-    executionObserver_ = observer;
+  void addExecutionObserver(ExecutionObserver* observer) {
+    executionObserverList_.push_back(*observer);
+  }
+
+  void removeExecutionObserver(ExecutionObserver* observer) {
+    executionObserverList_.erase(executionObserverList_.iterator_to(*observer));
   }
 
   /**
-   * Gets the execution observer associated with this EventBase.
+   * Gets the execution observer list associated with this EventBase.
    */
-  ExecutionObserver* getExecutionObserver() { return executionObserver_; }
+  ExecutionObserver::List& getExecutionObserverList() {
+    return executionObserverList_;
+  }
 
   /**
    * Set the name of the thread that runs this event base.
@@ -937,6 +960,7 @@ class EventBase : public TimeoutManager,
   LoopCallbackList loopCallbacks_;
   LoopCallbackList runBeforeLoopCallbacks_;
   Synchronized<OnDestructionCallback::List> onDestructionCallbacks_;
+  Synchronized<OnDestructionCallback::List> preDestructionCallbacks_;
 
   // This will be null most of the time, but point to currentCallbacks
   // if we are in the middle of running loop callbacks, such that
@@ -996,8 +1020,8 @@ class EventBase : public TimeoutManager,
   std::shared_ptr<EventBaseObserver> observer_;
   uint32_t observerSampleCount_;
 
-  // EventHandler's execution observer.
-  ExecutionObserver* executionObserver_;
+  // EventHandler's execution observer list (in case multiple are registered)
+  ExecutionObserver::List executionObserverList_;
 
   // Name of the thread running this EventBase
   std::string name_;
