@@ -94,6 +94,20 @@ struct compiler_must_not_predict_fn {
 FOLLY_INLINE_VARIABLE constexpr compiler_must_not_predict_fn
     compiler_must_not_predict{};
 
+// compiler_may_assume_separate_storage
+//
+// Informs the compiler that the given pointers point into regions that were
+// allocated separately (either in different globals, different stack variables,
+// different heap allocations, etc.), and that therefore any pointers at any
+// offset relative to one is never equal to any pointer at any offset relative
+// to the other.
+//
+// This can be helpful when some library has an ownership model that's opaque to
+// the compiler (e.g. modifying some_vector[0] never modifies the vector itself;
+// changes to parser state don't need to be stored to memory before reading the
+// next byte of data to be parsed, etc.).
+void compiler_may_assume_separate_storage(const void* a, const void* b);
+
 } // namespace folly
 
 #include <folly/lang/Hint-inl.h>
