@@ -61,6 +61,19 @@ AsyncLogWriter::~AsyncLogWriter() {
   folly::AtFork::unregisterHandler(this);
 }
 
+FOLLY_CONSTINIT std::atomic<AsyncLogWriter::DiscardCallback>
+    AsyncLogWriter::discardCallback_;
+
+void AsyncLogWriter::setDiscardCallback(DiscardCallback callback) {
+  discardCallback_.store(callback, std::memory_order_relaxed);
+}
+
+void AsyncLogWriter::invokeDiscardCallback(size_t numDiscarded) {
+  if (auto cb = discardCallback_.load(std::memory_order_relaxed)) {
+    (*cb)(numDiscarded);
+  }
+}
+
 void AsyncLogWriter::cleanup() {
   std::vector<std::string>* ioQueue;
   size_t numDiscarded;
@@ -75,6 +88,9 @@ void AsyncLogWriter::cleanup() {
     ioQueue = data->getCurrentQueue();
     numDiscarded = data->numDiscarded;
   }
+  if (numDiscarded > 0) {
+    invokeDiscardCallback(numDiscarded);
+  }
 
   // If there are still any pending messages, flush them now.
   if (!ioQueue->empty()) {
@@ -171,6 +187,10 @@ void AsyncLogWriter::ioThread() {
     // Write the log messages now that we have released the lock
     performIO(*ioQueue, numDiscarded);
 
+    if (numDiscarded > 0) {
+      invokeDiscardCallback(numDiscarded);
+    }
+
     // clear() empties the vector, but the allocated capacity remains so we can
     // just reuse it without having to re-allocate in most cases.
     ioQueue->clear();
