@@ -99,12 +99,11 @@ static_assert(
 /**
  * Log a message to this file's default log category, using a format string.
  */
-#define XLOGF(level, fmt, arg1, ...)       \
+#define XLOGF(level, fmt, ...)             \
   XLOG_IMPL(                               \
       ::folly::LogLevel::level,            \
       ::folly::LogStreamProcessor::FORMAT, \
       fmt,                                 \
-      arg1,                                \
       ##__VA_ARGS__)
 
 /**
@@ -112,13 +111,12 @@ static_assert(
  * predicate evaluates to true. Note that the condition is *only* evaluated
  * if the log-level check passes.
  */
-#define XLOGF_IF(level, cond, fmt, arg1, ...) \
-  XLOG_IF_IMPL(                               \
-      ::folly::LogLevel::level,               \
-      cond,                                   \
-      ::folly::LogStreamProcessor::FORMAT,    \
-      fmt,                                    \
-      arg1,                                   \
+#define XLOGF_IF(level, cond, fmt, ...)    \
+  XLOG_IF_IMPL(                            \
+      ::folly::LogLevel::level,            \
+      cond,                                \
+      ::folly::LogStreamProcessor::FORMAT, \
+      fmt,                                 \
       ##__VA_ARGS__)
 
 /**
@@ -162,7 +160,7 @@ static_assert(
  *
  * Note that this is threadsafe.
  */
-#define XLOGF_EVERY_MS(level, ms, fmt, arg1, ...)                        \
+#define XLOGF_EVERY_MS(level, ms, fmt, ...)                              \
   XLOGF_IF(                                                              \
       level,                                                             \
       [__folly_detail_xlog_ms = ms] {                                    \
@@ -172,7 +170,6 @@ static_assert(
         return folly_detail_xlog_limiter.check();                        \
       }(),                                                               \
       fmt,                                                               \
-      arg1,                                                              \
       ##__VA_ARGS__)
 
 namespace folly {
@@ -447,6 +444,25 @@ FOLLY_EXPORT FOLLY_ALWAYS_INLINE bool xlogFirstNExactImpl(std::size_t n) {
  */
 #define XLOG_IS_ON(level) XLOG_IS_ON_IMPL(::folly::LogLevel::level)
 
+/**
+ * Expects a fully qualified LogLevel enum value.
+ *
+ * This helper macro invokes XLOG_IS_ON_IMPL_HELPER() to perform the real
+ * log level check, with a couple additions:
+ * - If the log level is less than FOLLY_XLOG_MIN_LEVEL it evaluates to false
+ *   to allow the compiler to completely optimize out the check and log message
+ *   if the level is less than this compile-time fixed constant.
+ * - If the log level is fatal, this has an extra check at the end to ensure the
+ *   compiler can detect that it always evaluates to true.  This helps the
+ *   compiler detect that statements like XCHECK(false) never return.  Note that
+ *   XLOG_IS_ON_IMPL_HELPER() must still be invoked first for fatal log levels
+ *   in order to initialize folly::detail::custom::xlogFileScopeInfo.
+ */
+#define XLOG_IS_ON_IMPL(level)                              \
+  ((((level) >= ::folly::LogLevel::FOLLY_XLOG_MIN_LEVEL) && \
+    XLOG_IS_ON_IMPL_HELPER(level)) ||                       \
+   ((level) >= ::folly::kMinFatalLogLevel))
+
 /**
  * Helper macro to implement of XLOG_IS_ON()
  *
@@ -462,8 +478,8 @@ FOLLY_EXPORT FOLLY_ALWAYS_INLINE bool xlogFirstNExactImpl(std::size_t n) {
  *
  * See XlogLevelInfo for the implementation details.
  */
-#define XLOG_IS_ON_IMPL(level)                                  \
-  ((level >= ::folly::LogLevel::FOLLY_XLOG_MIN_LEVEL) && [] {   \
+#define XLOG_IS_ON_IMPL_HELPER(level)                           \
+  ([] {                                                         \
     static ::folly::XlogLevelInfo<XLOG_IS_IN_HEADER_FILE>       \
         folly_detail_xlog_level;                                \
     constexpr auto* folly_detail_xlog_filename = XLOG_FILENAME; \
