@@ -34,6 +34,12 @@ extern "C" FOLLY_KEEP void check_folly_function_nuke(void* fun) {
   static_cast<Function<void()>*>(fun)->~Function();
 }
 
+extern "C" FOLLY_KEEP void check_folly_function_move_assign(
+    void* src, void* dst) {
+  *static_cast<Function<void()>*>(src) =
+      std::move(*static_cast<Function<void()>*>(dst));
+}
+
 template <bool Triv, bool NxCopy>
 struct check_invocable_base;
 template <bool NxCopy>
@@ -245,14 +251,55 @@ static_assert(
     !std::is_constructible<Function<int const&() const>, int (*)()>::value, "");
 
 #if FOLLY_HAVE_NOEXCEPT_FUNCTION_TYPE
-static_assert(
-    !std::is_constructible<Function<int const&() noexcept>, int (*)()>::value,
-    "");
+static_assert( //
+    !std::is_constructible_v< //
+        Function<int() noexcept>,
+        int (*)()>);
 
-static_assert(
-    !std::is_constructible<Function<int const&() const noexcept>, int (*)()>::
-        value,
-    "");
+static_assert( //
+    !std::is_constructible_v< //
+        Function<int() const noexcept>,
+        int (*)()>);
+
+static_assert( //
+    std::is_constructible_v< //
+        Function<int() noexcept>,
+        int (*)() noexcept>);
+
+static_assert( //
+    std::is_constructible_v< //
+        Function<int() const noexcept>,
+        int (*)() noexcept>);
+
+static_assert( //
+    !std::is_constructible_v< //
+        Function<int const&() noexcept>,
+        int (*)()>);
+
+static_assert( //
+    !std::is_constructible_v< //
+        Function<int const&() const noexcept>,
+        int (*)()>);
+
+static_assert( //
+    !std::is_constructible_v< //
+        Function<int() noexcept>,
+        Function<int()>>);
+
+static_assert( //
+    !std::is_constructible_v< //
+        Function<int() const noexcept>,
+        Function<int() const>>);
+
+static_assert( //
+    std::is_constructible_v< //
+        Function<int()>,
+        Function<int() noexcept>>);
+
+static_assert( //
+    std::is_constructible_v< //
+        Function<int() const>,
+        Function<int() const noexcept>>);
 #endif
 
 static_assert(std::is_nothrow_destructible<Function<int(int)>>::value, "");
@@ -1276,7 +1323,7 @@ TEST(Function, SelfMove2) {
   Function<int()>& g = f;
   f = std::move(g);
   EXPECT_FALSE(bool(f)) << "self-assign is self-destruct";
-  EXPECT_EQ(0, alive) << "self-asign is self-destruct";
+  EXPECT_EQ(0, alive) << "self-assign is self-destruct";
   f = [] { return 43; };
   EXPECT_EQ(0, alive) << "sanity check against double-destruction";
   EXPECT_TRUE(bool(f));
